#!/usr/bin/python3
import re
from bs4 import BeautifulSoup
from concurrent.futures import ThreadPoolExecutor
import requests
from io import BytesIO
from PIL import Image
class ChapterNotFound(Exception):
    pass
class mangaObject:
    def __init__(self, json) -> None:
        self.json     = json
        self.chapters = json["chapter"][::-1]
        self.sinopsis = json["sinopsis"]
        self.author   = json["author"]
        self.status   = json["status"]
        self.genre    = json["genre"]
        self.released = json["released"]
        self.title    = json["title"]
        self.url      = json["url"]
        self.bytes    = BytesIO()

    def getLink(self, url):
        self.parse = BeautifulSoup(requests.get(url).text, 'html.parser')
        return [_.get('src').replace('https://cdn.statically.io/img/sekte/','https://') for _ in self.parse.find('div', id='readerarea').findAll('img')[1:]]

    # chapter: Integer
    def download(self, chapter, workers=5):
        assert isinstance(chapter, int)
        assert isinstance(workers, int)
        if self.bytes.getbuffer().tobytes():
            return self.bytes
        else:
            with ThreadPoolExecutor(max_workers=workers) as thread:
                result = []
                for i in self.getLink(self.chapters[chapter - 1]):
                    result.append(Image.open(BytesIO(thread.submit(requests.get, i).result().content)).convert("RGB"))
            result[0].save(fp=self.bytes,format='pdf', save_all=True, append_images=result[1:])
            return self.bytes

    def save_to_file(self, fn):
        assert isinstance(fn, str)
        if not self.bytes.getbuffer().tobytes():
            raise ChapterNotFound("Call download function first!")
        open(f"{fn}{'' if fn[-4:].lower() == '.pdf' else '.pdf'}", 'wb').write(self.bytes.getbuffer().tobytes())

    def __repr__(self):
        return f"<[{self.title}]>"

    def __str__(self) -> str:
        return self.__repr__()

class Search:
    def __init__(self, query) -> None:
        self.query = query
        self.req = requests.get("https://sektekomik.com/", params={"s": self.query}).text
        self.all_fetch = []

    @property
    def fetch(self):
        if self.all_fetch:
            return self.all_fetch
        title = re.findall("title=\"(.*?)\"", self.req)[3:][:-2]
        link = re.findall("<div class=\"bsx\"> <a href=\"(.*?)\"", self.req)
        thumb = [_.img['src'] for _ in BeautifulSoup(self.req, 'html.parser').findAll('div', class_='bs')]
        chapters = [_.replace('Chapter ','') for _ in re.findall("<div class=\"epxs\">(.*?)</div>", self.req)]
        for i in enumerate(thumb):
            self.all_fetch.append(Sekte(link[i[0]], i[1], title[i[0]], int(chapters[i[0]])))
        return self.all_fetch

    def __str__(self) -> str:
        return f"<[count {self.fetch.__len__()}]>"

    def __repr__(self) -> str:
        return self.__str__()


class Sekte:
    def __init__(self, link, thumb=None, title=None, chapters=None) -> None:
        self.link = link
        self.thumb  = thumb
        self.title  = title
        self.chapters = chapters

    def __str__(self) -> str:
        return self.title.__str__()

    def __repr__(self) -> str:
        return self.__str__()

    # chapt: Integer
    @property
    def manga(self):
        try:
            self.html = requests.get(self.link)
            parsing = BeautifulSoup(self.html.text, 'html.parser')
            komik = [_.a.get('href') for _ in parsing.find('div', id='chapterlist').findAll('div', class_='eph-num')]
            return mangaObject(
                {
                    'title': self.title,
                    'chapter': komik,
                    'url': self.html.url,
                    'sinopsis': parsing.find('div', attrs={'itemprop': 'description'}).text,
                    'genre': [_.text for _ in parsing.find('div', class_='bigcontent').findAll('a', rel='tags')],
                    'status': re.search('Status <i>(.*?)</i>', self.html.text).group(1),
                    'released': parsing.find('div', class_='fmed').span.text.strip(),
                    'author': parsing.find('div', class_='fmed').nextSibling.span.text.strip()
                }
            )
        except IndexError:
            raise ChapterNotFound("Error")



import sys
from .Dwrapper import nhentai
import os
import argparse
name=os.path.dirname(__file__).split('/')[-1]
parser=argparse.ArgumentParser(prog=f"python -m {name}", description="doujin wrapper")
parser.add_argument("--nuklir", help="Nhentai Code")
parser.add_argument("--output", help="File")
args=parser.parse_args()
if args.nuklir:
    try:
        d=nhentai(args.nuklir).doujin
        print(f"title: {d.title}")
        print(f"number of sheets: {d.images.__len__()} sheet")
        sys.stdout.write("\rstatus: Dowloading ...")
        d.save_to_file(args.output if args.output else d.title)
        filename = args.output or d.title
        sys.stdout.flush()
        sys.stdout.write("\rstatus: Dowloaded      \n")
        sys.stdout.flush()
        print(f"Save as {filename}{'' if filename[-4:]=='.pdf' else '.pdf'}")
    except Exception:
        print("Invalid Code")
else:
    print(f"Try `python -m {name} -h' for more information.")



from setuptools import setup, find_packages
from os import path
base_dir = path.abspath(path.dirname(__file__))
setup(
  name = 'sekte2pdf',
  packages = ['sekte'],
  include_package_data=True,
  version = '0.0.3',    
  license='MIT',
  description = 'Sekte2Pdf',
  long_description_content_type = 'text/markdown',
  long_description = open('README.md', 'r').read(),
  author = 'MhankBarBar', # Krypton-Byte
  author_email = 'mhankbarbar@yes.my',
  url = 'https://github.com/MhankBarBar/SekteModule',
  download_url = 'https://github.com/MhankBarBar/SekteModule/archive/0.0.3.tar.gz',
  keywords = ['manga', 'sektekomik', 'downloader', 'pdf', 'sektekomik downlader'], 
  install_requires=[           
          'pillow',
          'requests',
          'bs4'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      
    'Intended Audience :: Developers',      
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',  
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
  ],
)
